﻿using Lab1.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Lab1.Drawnings;

public class DrawningTruck
{
    /// <summary>
    /// Класс-сущность
    /// 
    /// </summary>
    public EntityTruck? EntityTruck { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки автомобиля
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя кооридната прорисовки автомобиля
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки автомобиля
    /// </summary>
    private readonly int _drawningCarWidth = 140;

    /// <summary>
    /// Высота прорисовки автомобиля
    /// </summary>
    private readonly int _drawningCarHeight = 70;

    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningCarWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningCarHeight;


    public DrawningTruck()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningTruck(int speed, int weight, Color bodyColor) : this()
    {
        EntityTruck = new EntityTruck(speed, weight, bodyColor);

    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningCarWidth">Ширина прорисовки автомобиля</param>
    /// <param name="_drawningCarHeight">Высота прорисовки автомобиля</param>
    protected DrawningTruck(int drawningCarWidth, int drawningCarHeight) : this()
    {
        _drawningCarWidth = drawningCarWidth;
        _pictureHeight = drawningCarHeight;

    }

    /// <summary>
	/// Установка границ поля
	/// </summary>
	/// <param name="width">Ширина поля</param>
	/// <param name="height">Высота поля</param>
	/// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
	public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (_drawningCarWidth <= width || _drawningCarHeight <= height)
        {
            _pictureWidth = width;
            _pictureHeight = height;
            if (_startPosX != null && _startPosY != null)
                if (_startPosX + _drawningCarWidth > _pictureWidth)
                    _startPosX = _pictureWidth - _drawningCarWidth;
            if (_startPosY + _drawningCarHeight > _pictureHeight)
                _startPosY = _pictureHeight - _drawningCarHeight;
            return true;
        }
        else
            return false;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах
        if (x < 0)
            x = 0;
        else if (x + _drawningCarWidth > _pictureWidth)
            x = _pictureWidth.Value - _drawningCarWidth;
        if (y < 0)
            x = 0;
        else if (y + _drawningCarHeight > _pictureHeight)
            y = _pictureHeight.Value - _drawningCarHeight;

        _startPosX = x;
        _startPosY = y;


    }
    /// <summary>
	/// Изменение направления перемещения
	/// </summary>
	/// <param name="direction">Направление</param>
	/// <returns>true - перемещене выполнено, false - перемещение невозможно</returns>
	public bool MoveTransport(DirectionType direction)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityTruck.Step > 0)
                {
                    _startPosX -= (int)EntityTruck.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTruck.Step > 0)
                {
                    _startPosY -= (int)EntityTruck.Step;
                }
                return true;
            // вправо
            case DirectionType.Right:
                //TODO прописать логику сдвига в право
                if (_startPosX.Value + EntityTruck.Step < _pictureWidth - _drawningCarWidth)
                {
                    _startPosX += (int)EntityTruck.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                //TODO прописать логику сдвига в вниз
                if (_startPosY.Value + EntityTruck.Step < _pictureHeight - _drawningCarHeight)
                {
                    _startPosY += (int)EntityTruck.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }


      
        
        Brush br = new SolidBrush(EntityTruck.BodyColor);

        //кузов уборщика
        g.FillRectangle(br, _startPosX.Value + 10, _startPosY.Value + 30, 140, 20);
        g.FillRectangle(br, _startPosX.Value + 115, _startPosY.Value, 35, 30);
        //колеса уборщика
        g.FillEllipse(br, _startPosX.Value + 15, _startPosY.Value + 50, 25, 25);
        g.FillEllipse(br, _startPosX.Value + 120, _startPosY.Value + 50, 25, 25);
        g.FillEllipse(br, _startPosX.Value + 50, _startPosY.Value + 50, 25, 25);

    }
}
