﻿using Lab1.Drawnings;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Windows.Forms.VisualStyles.VisualStyleElement.TrackBar;

namespace Lab1.MovementStrategy;

public class MoveableTruck : IMoveableObjectcs
{
    /// <summary>
	/// Поле-объект класса DrawningCar или его наследника
	/// </summary>
	private readonly DrawningTruck? _truck = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="car">Объект класса DrawningCar</param>
    public MoveableTruck(DrawningTruck truck)
    {
        _truck = truck;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_truck == null || _truck.EntityTruck == null || !_truck.GetPosX.HasValue || !_truck.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_truck.GetPosX.Value, _truck.GetPosY.Value, _truck.GetWidth, _truck.GetHeight);
        }
    }

    public int GetStep => (int)(_truck?.EntityTruck?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_truck == null || _truck.EntityTruck == null)
        {
            return false;
        }

        return _truck.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
        };
    }
}
